
// The "optimized" RC6 source appears to be buggy, and causes a crash in some cases, since I didn't want to touch
//	any of the crypto code, I didn't try to fix it.  Instead, I've included a different "unoptimized" implementation 
#define RC6_unopd	1 	// set to 0 for the optimized (buggy) version of "rc6_key"

#define kCryptKeyPSize		64
#define kCryptKeyLMaxSize	32
#define kCryptKeyLMinSize	5
#define kCryptKeyHexMin	5	// what is the min key len that will cause user supplied key to be padded and converted into hex representation
#define kCryptKeyHexMax	8	// what is the max key len that will cause user supplied key to be padded and converted into hex representation

#define kDecryptedBufferPSize		32000
#define kEncryptedBufferPSize		64000
#define kDecryptedBufferLSize		30000 	// Max uncoded data size: kDecryptedBufferLSize characters
#define kEncryptedBufferLSize		60000
#define kDecryptedBufferMinSize 	6 		// Min uncoded data size: kDecryptedBufferMinSize

#define kRC4ChunkSize		256
#define kRC4ChunkSizeBits	8

#define kRC4KeySize	256 // ints

#define kRC6ChunkSize		16
#define kRC6ChunkSizeBits	4

#define kRC6KeySize	43 // longs

#define kROTencode	(unsigned char)0
#define kROTdecode	(unsigned char)1


static void crypto_StrToHex(unsigned char *theSource, unsigned long tSourceSize, unsigned char *theResult, unsigned long *tResultSize);
static void crypto_HexToStr(unsigned char *theSource, unsigned long tSourceSize, unsigned char *theResult, unsigned long *tResultSize);

static void crypto_ROT(unsigned char rotValue, unsigned char rotDirection,  unsigned long buffSize, unsigned char *buffer);



// -=-=- "RC4 COMPATIBLE" -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

/*! \file rc4.h
    \brief Header file for RC4 stream cipher routines
	 \author Damien Miller <djm@mindrot.org> 	\version 0.0.0	 \date 1999	 
	 A simple implementation of the RC4 stream cipher, based on the
	 description given in _Bruce Schneier's_ "Applied Cryptography"
	 2nd edition.
    This code is licensed under the GNU GPL. Please see the file
    COPYING for details
	 \warning The name "RC4" is trademarked in the United States, you may need to use "RC4 compatible" or "ARC4"  (Alleged RC4).
*/
#ifndef __RC4TYPE__
#define __RC4TYPE__ 1
typedef struct
{
	unsigned int s[kRC4KeySize];
	int i;
	int j;
} rc4_t;
#endif

/*! \fn void rc4_key(rc4_t *r, unsigned char *key, int len);
    \brief Set up key structure of RC4 stream cipher
	 \param r pointer to RC4 structure to be seeded
	 \param key pointer to buffer containing raw key
	 \param len length of key
	 
	 This function set the internal state of the RC4 data structure
	 pointed to by \a r using the specified \a key of length \a len.
	 
	 This function can use up to 256 bytes of key, any more are ignored.
	 
	 \warning Stream ciphers (such as RC4) can be insecure if the same
	 \warning key is used repeatedly. Ensure that any key specified has
	 \warning an reasonably sized Initialisation Vector component.
*/
static void rc4_key(rc4_t *r, unsigned char *key, int len);

/*! \fn rc4_crypt(rc4_t *r, unsigned char *plaintext, int len);
    \brief Crypt bytes using RC4 algorithm
	 \param r pointer to RC4 structure to be used
	 \param plaintext Pointer to bytes to encrypt
	 \param len number of bytes to crypt

	 This function encrypts one or more bytes (pointed to by \a plaintext)
	 using the RC4 algorithm. \a r is a state structure that must be 
	 initialiased using the rc4_key() function prior to use.
	 
	 Since RC4 XORs each byte of plaintext with a byte of keystream,
	 this function can be used for both encryption and decryption.
*/
static void rc4_crypt(rc4_t *r, unsigned char *plaintext, int len);

/*! \fn rc4_getbytes(rc4_t *r, unsigned char *buffer, int len);
    \brief Generate key stream using the RC4 stream cipher
	 \param r pointer to RC4 structure to be used
	 \param buffer pointer to buffer in which to deposit keystream
	 \param len number of bytes to deposit

	 This function gives access to the raw RC4 key stream. In this 
	 consiguration RC4 can be used as a fast, strong pseudo-random 
	 number generator with a very long period.
*/
static void rc4_getbytes(rc4_t *r, unsigned char *buffer, int len);



// -=-=- "RC6 COMPATIBLE" -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// -=-=- "RC6 COMPATIBLE" -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// -=-=- "RC6 COMPATIBLE" -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

/* This is an independent implementation of the encryption algorithm:   */
/*         RC6 by Ron Rivest and RSA Labs                               */
/* which is a candidate algorithm in the Advanced Encryption Standard   */
/* programme of the US National Institute of Standards and Technology.  */
/* Copyright in this implementation is held by Dr B R Gladman but I     */
/* hereby give permission for its free direct or derivative use subject */
/* to acknowledgment of its origin and compliance with any conditions   */
/* that the originators of the algorithm place on its exploitation.     */
/* Dr Brian Gladman (gladman@seven77.demon.co.uk) 14th January 1999     */
/* Timing data for RC6 (rc6.c)
128 bit key:
Key Setup:    1632 cycles
Encrypt:       270 cycles =    94.8 mbits/sec
Decrypt:       226 cycles =   113.3 mbits/sec  
Mean:          248 cycles =   103.2 mbits/sec
*/
#ifndef __RC6TYPE__
#define __RC6TYPE__ 1
typedef unsigned long	rc6_t[kRC6KeySize];  /* storage for the RC6 key schedule  */

/* Rotation operators. x must be unsigned, to get logical right shift*/
#define ROTL(x,y) (((x)<<(y&(31))) | ((x)>>(32-(y&(31)))))
#define ROTR(x,y) (((x)>>(y&(31))) | ((x)<<(32-(y&(31)))))
#endif

// Setup the key...
#if RC6_unopd
// 	RC6_unopd code based on public domain sample implementation by Salvo Salasio
	static void rc6_key(rc6_t l_key, unsigned char *in_key, const unsigned int key_len);
#else
/*         RC6 by Ron Rivest and RSA Labs                               */
	static void rc6_key(rc6_t l_key, unsigned long *in_key, const unsigned long key_len);
#endif

/* encrypt a block of text  */  // DOES ONLY 16 BYTES AT A TIME!!!  Text must be in blocks of 16 bytes!
static void rc6_encrypt(rc6_t l_key, const unsigned long *in_blk, unsigned long *out_blk);

/* decrypt a block of text  */ // DOES ONLY 16 BYTES AT A TIME!!!  Text must be in blocks of 16 bytes!
static void rc6_decrypt(rc6_t l_key, const unsigned long *in_blk, unsigned long *out_blk);






