/*  Metrowerks Standard Library  Version 2.4  1998 March 10  */

/*  $Date:: 3/10/98 7:38 PM                                  $ 
 *  $Revision:: 10                                           $ 
 *  $NoKeywords: $ 
 *
 *		Copyright  1995-1998 Metrowerks, Inc.
 *		All rights reserved.
 */

/*
 *	console_io.mac.c
 *	
 *	
 *	Routines
 *	--------
 *		__read_console
 *		__write_console
 *		__close_console
 *
 *
 */

// #include <SIOUX.h>
// #include <console.h>
#include <abort_exit.h>
#include <console_io.h>
#include <critical_regions.h>
#include <misc_io.h>

enum console_status {
	no_console,
	console_open,
	console_broken
};

static int console_status = no_console;

static int check_console(void)
{
	int	result = 0;
	
	if (console_status == console_open)
		return(1);
	
	__begin_critical_region(console_status_access);
	
	if (console_status != no_console)
		goto exit;
	
	__stdio_atexit();
	
	if (InstallConsole(0))
	{
		console_status = console_broken;
		goto exit;
	}
	__console_exit = RemoveConsole;
	
	console_status = console_open;
	
	result = 1;
	
exit:
	
	__end_critical_region(console_status_access);
	
	return(result);
}

int __read_console(__file_handle handle, unsigned char * buffer, size_t * count, __idle_proc idle_proc)
{
#pragma unused(handle,idle_proc)

	if (!check_console())
		return(__io_error);
	fflush(stdout);                   /* mm-960717*/
	*count = ReadCharsFromConsole((char *) buffer, *count);
	
	if (*count == -1)
		return(__io_error);
	
	return(__no_io_error);
}

int __write_console(__file_handle handle, unsigned char * buffer, size_t * count, __idle_proc idle_proc)
{
#pragma unused(handle,idle_proc)

	if (!check_console())
		return(__io_error);
	
	*count = WriteCharsToConsole((char *) buffer, *count);
	
	if (*count == -1)
		return(__io_error);
	
	return(__no_io_error);
}

int __close_console(__file_handle handle)
{
#pragma unused(handle)

	return(__no_io_error);
}

/*  Change Record
 *	01-Sep-95 JFH  First code release.
 *  15-Dec-95 JFH  Added installation of hook to close console so that RemoveConsole isn't
 *								 called unconditionally at _exit() time.
 *  96-Jul-17 MM   Added flush to make sure that characters awaiting to appear on console
 *                               are output before reading is attemmpted.
*/
