// "PUBLIC" Routines  -  These will be generic, generally non-specific functions that every plugin has.

/*:::::::::::::::::::::::::::  Standard PlugIn Routines   :::::::::::::::::::::::::::::*/


// Recieved a message to initialize the plugin, so do one-time init and idle init routines.
static long Do_Init(short version) {
	
#if FM_CPU_X86
	char regPath[256];
	char tempValue[256];
	PHKEY myRegKey;
	LPDWORD createKeyResult;
	DWORD ckResult;
	long regKeyResult;
	LPDWORD regKeyValueType;
	DWORD rkvType;
	LPBYTE regKeyValue;
	LPDWORD regKeyValueLength;
	DWORD rkvLength;
	struct lconv *localeInfoP;
//	struct lconv localeInfo;
	unsigned char tempPrefsLen;
#else
	Intl0Rec	intlResourceRec;
	OSErr		macOSerr;
	short		vRefNum;
	OSType		folderType;
	Boolean		createFolder;
	short		fRefNum;
	FSSpec		prefsFileFSSpec;
	long		prefFileLineLen;
#endif

// check version of API that this plugin was compiled with, to the API vers of the actual application
if ((version < kMinExtnVersion) || (version > kMaxExtnVersion)) 
{
		#if  DEBUG_VERSION
				DisplayErrorNumber(kErrorMsg, version, 12);
		#endif

	return kBadExtnVersion;   // do not load plugin if the API vers is not compatible
};

//	GetDateTime((unsigned long*) &qd.randSeed);


// allocate room for a temp crypting buffer...
gCryptBufferSize = kEncryptedBufferPSize;
gCryptBuffer = FMX_NewHandle(gCryptBufferSize);
if (FMX_MemoryError() != 0) 
{
	#if FM_CPU_X86
		DisplayMessageString(kErrorMsg, "Warning out of memory, please give more to FMP.\0", 23401);
	#else
		DisplayMessageString(kErrorMsg, "\pWarning out of memory, please give more to FMP.", 23401);
	#endif
	return kDoNotEnable;   // do not load plugin if the API vers is not compatible
}
else
{
	FMX_MoveHHi(gCryptBuffer);
	FMX_LockHandle(gCryptBuffer);
		
	// clear out our cryptkey storage
	zeroCleanMemory(gCryptKey, kCryptKeyPSize);
	gCryptKeyLen = 0;

};

#if FM_CPU_X86 // use Windows way of getting/storing prefs
localeInfoP = localeconv();
decimalSepCH = *(localeInfoP->decimal_point);  // for reals, as in "3.1415"
decimalSepMonCH = decimalSepCH;  // for reals, monetary values
largeSepCH= *(localeInfoP->thousands_sep);      // for large numbers,  as in "2,122,433"
largeSepMonCH = largeSepCH;      // for large numbers, monetary values


	// read registery prefs or sets defaults & creates reg entry

ckResult = 0;
createKeyResult= &ckResult;

rkvType = 0;
regKeyValueType= &rkvType;

rkvLength = 0;
regKeyValueLength = &rkvLength;

tempValue[0] = 0;
regKeyValue = (unsigned char *)tempValue;

lstrcpy( (LPTSTR)regPath, k_Reg_Path);  // copy the standard path (+ the format string value)  to a string
// open/create the appropriate key, store the result in myRegKey,   createKeyResult contains either a "created" or "opened" value.
regKeyResult = RegCreateKeyEx (HKEY_CURRENT_USER, (const char *)regPath, 0, "REG_SZ\0", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS , NULL, (PHKEY)&myRegKey, createKeyResult);

if ( regKeyResult != 0 )  // some sort of error
	{
	DisplayErrorNumber(kErrorMsg, regKeyResult, 107001);   // 107001 is just an arbitrary number to help me
											  //    locate the source of an error message in the
											  //	  source code when a user reports an error code.
	return kDoNotEnable;
	};

if ((*createKeyResult) == REG_CREATED_NEW_KEY)  // this is the first time this plugin has initialized, or the key was missing
	{
		pref_formatString[0] = '1';
		pref_formatString[1] = 0;
		
		(*regKeyValueLength) = 1;

	regKeyResult = RegSetValueEx ( myRegKey, k_Reg_Key_Format, 0, REG_SZ, (CONST BYTE *)pref_formatString, (DWORD)(*regKeyValueLength));
	
	if ( regKeyResult != 0 )
		{
		DisplayErrorNumber(kErrorMsg, regKeyResult, 107002);   
		return kDoNotEnable;
		};
	};

// query the key to see how big the value is.
regKeyResult = RegQueryValueEx (myRegKey, k_Reg_Key_Format, NULL, NULL, NULL, regKeyValueLength);

if  ( (*regKeyValueLength) > 21)  // format string should only be 20 chars ( + 1 char for length byte / null terminator )
	{
	DisplayErrorNumber(kErrorMsg, regKeyResult, 107003);   
	return kDoNotEnable;
	};
	
// query the key to get the value.
regKeyResult = RegQueryValueEx (myRegKey, k_Reg_Key_Format, NULL, NULL, regKeyValue, regKeyValueLength);

if ( regKeyResult != 0 )
		{
		DisplayErrorNumber(kErrorMsg, regKeyResult, 107004);   
		return kDoNotEnable;
		};
			
regKeyValue[(*regKeyValueLength)] = 0;
lstrcpy( pref_formatString, (const char *)regKeyValue);   // copy the contents of the key to our internal format string

RegCloseKey (myRegKey);

tempPrefsLen = lstrlen(pref_formatString);

pref_formatString[tempPrefsLen+1] = 0;  // make sure the regKeyValue stored in the format string is null terminated

#else  // use Macintosh way of getting/storing prefs
	intlResourceRec = (**((Intl0Hndl)GetIntlResource(0)));  // get the international compatible settings for
										          			//    numbers (decimal, currency, etc...)
	decimalSepCH = intlResourceRec.decimalPt;  // for reals, as in "3.1415"
	decimalSepMonCH = decimalSepCH;
	largeSepCH= intlResourceRec.thousSep;      // for large numbers,  as in "2,122,433"
	largeSepMonCH = largeSepMonCH;

	// obtain where the prefs file should be
	vRefNum = kOnSystemDisk;  // system volume of the user's machine
	folderType = kPreferencesFolderType; // look for the proper folder for preferences
	createFolder = kCreateFolder; // create folder if it doesn't already exist
	
	// get the disk/folder IDs
	macOSerr = FindFolder( vRefNum, folderType, createFolder, &(prefsFileFSSpec.vRefNum), &(prefsFileFSSpec.parID) ); 
	if (macOSerr != 0) // if there was some problem at this point, we want to avoid loading the plugin
		{
		DisplayErrorNumber(kErrorMsg, (int)macOSerr, 107001);   // 107001 is just an arbitrary number to help me
														 	//    locate the source of an error message in the
														 	//	  source code when a user reports an error code.
		return kDoNotEnable;
		};
	
	// read pref file or sets defaults & creates file
	
	// set the filename
	prefsFileFSSpec.name[0] = 6; // set length

	prefsFileFSSpec.name[1] = k_mRsrc1Creator;  
	prefsFileFSSpec.name[2] = k_mRsrc2Creator;  
	prefsFileFSSpec.name[3] = k_mRsrc3Creator; 
	prefsFileFSSpec.name[4] = k_mRsrc4Creator;  
	prefsFileFSSpec.name[5] = '.'; 
	prefsFileFSSpec.name[6] = (unsigned char)'';

	macOSerr = FSpOpenDF( &prefsFileFSSpec, fsCurPerm, &fRefNum ); // try to open the file, to see if it exists
	if (  (macOSerr != 0 ) && (macOSerr != -43 ))
		{  // if there is some other error, besides file not found, do no load plugin
		DisplayErrorNumber(kErrorMsg, (int)macOSerr, 107002);
		return kDoNotEnable;
		};
		
	if (macOSerr == -43)
		{  // the file was not found, so create it using the built in prefs
		pref_formatString[0] =1;  // length of the default prefs

		pref_formatString[1] = '1';

		
		macOSerr =FSpCreate( &prefsFileFSSpec, k_mRsrcCreator, 'TEXT', smSystemScript );
		if ( macOSerr != 0)
			{  // if there is some other error do not load plugin
			DisplayErrorNumber(kErrorMsg, (int)macOSerr, 107003);
			return kDoNotEnable;
			};

		macOSerr =FSpOpenDF( &prefsFileFSSpec, fsCurPerm, &fRefNum ); // try again to open the file
		if ( macOSerr != 0)
			{  // if there is some other error do not load plugin
			DisplayErrorNumber(kErrorMsg, (int)macOSerr, 107004);
			return kDoNotEnable;
			};

		// write the defaults
		prefFileLineLen = pref_formatString[0]+1; // write out the whole string, including the leading length byte
		macOSerr =FSWrite( fRefNum, (long *) &prefFileLineLen, pref_formatString ); 
		if ( macOSerr != 0)
			{  // if there is some other error do not load plugin
			DisplayErrorNumber(kErrorMsg, (int)macOSerr, 107005);
			return kDoNotEnable;
			};
		
		// reset the file position so that when we read the preferences, we will get the default we just
		//   wrote to this file.
		SetFPos( fRefNum, fsFromStart, 0 );  // beginning of file
		};
		
		
		prefFileLineLen = 21; // we only support a length of 20 chars for our format string (+1 for length byte)
		macOSerr = FSRead( fRefNum,(long *)&prefFileLineLen, pref_formatString );
		if ( (macOSerr != 0) && (macOSerr != -39) )  // format string might be <21 and cause an EOF error
			{  // if there is some other error, besides EOF,  do no load plugin
			DisplayErrorNumber(kErrorMsg, (int)macOSerr, 107006); 
			return kDoNotEnable;
			};
		
		FSClose(fRefNum);

		prefFileLineLen = pref_formatString[0];


		#if  DEBUG_VERSION
		#endif
				
#endif

	return kCurrentExtnVersion;
}



// Recieved a message that the user wishes to configure this plugin, so display the configuration dialog
//    NOTE: this message will only need to be handled if we set the idle flag in the feature string
static void Do_AppPrefs(void) {
#if FM_CPU_X86
	int dlgResult;
	HWND	topWind;

	topWind = GetActiveWindow();  // get the active window (the plugin window)
	dlgResult = DialogBox((HINSTANCE)gFMExternCallPtr->instanceID,MAKEINTRESOURCE(PREF_DIALOG), topWind, (DLGPROC)prefProc);
	if (dlgResult == -1)  // if we couldn't pull up the dialog....
		{ // and this is the debug version of the plugin....
#if  DEBUG_VERSION
		DisplayErrorNumber(kErrorMsg, GetLastError(), 110001);
#endif
		};
#else
	Str255	tempstring;
	short fmOriginalResPath = CurResFile();
	DialogPtr configPrefsDialog;
	short itemHit;
	Handle itemHandle;
	GrafPtr		oldPort;
	short	x;
	short 			itemType;  // for button hiliting
	Rect 			itemRect;
	Boolean			hasFormatSymbols;

	// make sure we are in the right resource fork context
	UseResFile(pb_resourceID);

	// allocate the dialog, save the previous grafport, set the grafport to our dialog
	configPrefsDialog = GetNewDialog( PREF_DIALOG, nil, (WindowPtr)-1 );
	GetPort(&oldPort);
	SetPort(configPrefsDialog);

	// draw a border around the ok button
	GetDialogItem(configPrefsDialog, 1, &itemType, &itemHandle, &itemRect);
	InsetRect( &itemRect, -4, -4);
	PenSize(3,3);
	FrameRoundRect(&itemRect, 16, 16);
	PenNormal();

	// put the contents of temp string into the item #3 (edit text) as a copy the user can manipulate
	GetDialogItem( configPrefsDialog, 3, &itemType, &itemHandle, &itemRect );
	SetDialogItemText( itemHandle, (const unsigned char *)pref_formatString );

	// process the dialog (user interaction here)
	for (itemHit = 0; ((itemHit != kStdOkItemIndex) && (itemHit != kStdCancelItemIndex));)
		{
		ModalDialog( nil, &itemHit);
		switch (itemHit)
			{
			case kStdOkItemIndex: // item 1 - ok button
				// get text from item 3 (edit text box) and copy it to temp string
				GetDialogItem( configPrefsDialog, 3, &itemType, &itemHandle, &itemRect );
				GetDialogItemText( itemHandle, tempstring );
				
				// see if the contents of item 3 (the user provided format string) contains a format symbol #
				hasFormatSymbols = false;
				for (x = 1; x <= tempstring[0];x++)
					{
					if (tempstring[x] == '#')
						hasFormatSymbols = true;
					};
				
				// test item text to make sure it's a valid format for our function.  it must have something in it
				//    it must not be bigger than the max size for the format string, and it must have format symbols
				if ( (tempstring[0] > 0) && (tempstring[0] <= k_maxSizeFormat) && hasFormatSymbols)
					{ // copy the contents of the temp string into the format string
					BlockMoveData(tempstring, pref_formatString, tempstring[0]+1);
					}
				    else
				    { // give an error beep, reset the contents of item 3 to the format string
				      //    select the text within item 3, and ignore the pressing of the ok button
				    SysBeep(1);
				    SetDialogItemText( itemHandle, (const unsigned char *)pref_formatString );
				    SelectDialogItemText( configPrefsDialog, 3, 0, 32767 );
				    itemHit = 0;
				    };
				break;
			
			case kStdCancelItemIndex: // item 2 - cancel button
				// restore all preferences to their original values, and exit dialog
				break;
			};  // switch itemHit
		}; // for itemHit
	
	SetPort( oldPort);
	DisposeDialog(configPrefsDialog);
	
	UseResFile(fmOriginalResPath);
	
	writePrefs ();
	
#endif
}




static void Do_Idle(FChar unsafeCalls) {

/* We are going to pretend that the "Something" code needs to make some high level calls */
	if (!unsafeCalls) {
		// idle functions
	}
}



static void Do_Shutdown(void) 
{
FMX_UnlockHandle(gCryptBuffer);
FMX_DisposeHandle(gCryptBuffer);
} // Do_Shutdown

